(function( $ ) {
	'use strict';

	/**
	 * All of the code for your admin-facing JavaScript source
	 * should reside in this file.
	1. Initiate install
	2. Initiate Data load
	3. Reset install
	4. Set Start and End dates
	5. Error reporting

	*/


  

	/*
	Option to reset full install and start over.
	*/	

	jQuery('#icdash_activate_install_again').on
	(
		{
		 click: function(e) 
			{
				if (confirm("Are your sure you want to reset and reinstall?"))
				{
					jQuery('.icdash_after_install').hide(); 
					jQuery('#icdash_start_install').show();
				}
			}
		}	
	);

	/*
	Start the Full Install. It loads the scripts necessary and for each script it uses server side events to keep the user informed
	*/	
	jQuery('#icdash_start_install').on
	(
		{

			click: function(e) 
			{
				jQuery( "#icdash-tabs" ).tabs(  "option", "active", 1 );
				jQuery("#icdash-progress-parent").show("slow");
			
		
				var es;
		
				var v_urls=icdash_global["script_url"];		 //these urls are passed by the plugin main page that generates the config page  
				var v_current_url=0;	
				var startTime = new Date();
				var endTime=new Date();
				startTask();
				clearLog("Processing...");
				
				function startTask() {
					
					
					$("#result_header").html("Processing script # "+(v_current_url+1));
					addLog("Processing script # "+(v_current_url+1)); 
					
					es = new EventSource(v_urls[v_current_url]);	  
		
					//a message is received
					es.addEventListener('message', function(e) {
					
						var result = JSON.parse( e.data );
						  
						if (e.lastEventId == 'HEADER') 
						{
							$("#result_header").html(result.message);
						}				  
						
						addLog(result.message+' '+result.url);       
						  
						if(e.lastEventId == 'CLOSE') 
						{
							//addLog('Received CLOSE closing');
							es.close();
		
							
							//now start another URL
							v_current_url++;
							if (v_current_url<v_urls.length) 
							{
								startTask();
							}
							else //all scripts installed and complete
							{
								jQuery("#result_header").html("Process complete");
								
								$( "#icdash_dialog" ).dialog({
								  modal: true,
								  buttons: {
									Ok: function() {
									  $( this ).dialog( "close" );
									}
								  }
								});	
								jQuery(".icdash_after_install").show("slow");
								jQuery('#icdash_start_install').hide("slow");
							}	
							
						}
						else if(e.lastEventId =='URL_TRANSFER')
						{
						  es.close();
		
							setTimeout(function () {
								window.location.href= result.url; // the redirect goes here
							},2000); // 5 seconds
							
						}
						else if(e.lastEventId == 'START') {
							addLog('Start');
							
						}
						
						else {
							/*Do Nothing: Ideally this should not happen*/
						}
					});
					  
					es.addEventListener('error', function(e) 
					{
						addLog('Error occurred');
						endTime=new Date();
						var timeDiff = endTime - startTime; //in ms
						// strip the ms
						timeDiff /= 1000;	   
						addLog('The script took longer than '+timeDiff+" seconds. ");
						addLog('Possibly your PHP session has timed out. Please adjust or increase your timeout and restart the install ');
						addLog('<strong>Troubleshoot:</strong> Check log files in the <strong>infocaptor/logs</strong> subdirectory and send them to contact@infocaptor.com');
						console.log(e);
						es.close();
					});
				}
				  
				function stopTask() {
					es.close();
					addLog('Interrupted');
				}
				  
				function addLog(message) {
					var r = document.getElementById('results');
					r.innerHTML += message + '<br>';
					r.scrollTop = r.scrollHeight;
				}		
				function clearLog(message) {
					var r = document.getElementById('results');
					r.innerHTML = message + '<br>';
					r.scrollTop = r.scrollHeight;
				}
		
			}

		}
	);
	

	
	jQuery('#icdash_start_upgrade').on
	(
		{

			click: function(e) 
			{
				
				jQuery( "#icdash-tabs" ).tabs(  "option", "active", 1 );
				jQuery("#icdash-progress-parent").show("slow");

				var es;
		
				var v_urls=icdash_global["download_url"];		 //these urls are passed by the plugin main page that generates the config page  
				var v_current_url=0;		   
				startTask();
				clearLog("Processing...");
				
				function startTask() {
					
					
					$("#result_header").html("Processing script # "+(v_current_url+1));
					addLog("Processing script # "+(v_current_url+1)); 
					
					es = new EventSource(v_urls[v_current_url]);	  
					
		
					//a message is received
					es.addEventListener('message', function(e) {
					
						var result = JSON.parse( e.data );
						  
						if (e.lastEventId == 'HEADER') 
						{
							$("#result_header").html(result.message);
						}				  
						
						addLog(result.message+' '+result.url);       
						  
						if(e.lastEventId == 'CLOSE') 
						{
							
							es.close();
		
							
							//now start another URL
							v_current_url++;
							if (v_current_url<v_urls.length) 
							{
								startTask();
							}
							else //all scripts installed and complete
							{
								jQuery("#result_header").html("Process complete");
								jQuery('#icdash_start_upgrade').hide();
								//jQuery("#icdash-dashboard-list").show("slow");
							}	
							
						}
						else if(e.lastEventId =='URL_TRANSFER')
						{
						  es.close();
		
							setTimeout(function () {
								window.location.href= result.url; // the redirect goes here
							},2000); // 5 seconds
							
						}
						else if(e.lastEventId == 'START') {
								addLog("Start");
						}
						
						else {
		
						}
					});
					  
					es.addEventListener('error', function(e) {
						addLog('Error occurred');
						addLog('Possibly your PHP session has timed out. Please adjust or increase your timeout and restart the install ');
						addLog('<strong>Troubleshoot:</strong> Check log files in the <strong>infocaptor/logs</strong> subdirectory and send them to contact@infocaptor.com');
						console.log(e);				
						es.close();
					});
				}
				  
				function stopTask() {
					es.close();
					addLog('Interrupted');
				}
				  
				function addLog(message) {
					var r = document.getElementById('results');
					r.innerHTML += message + '<br>';
					r.scrollTop = r.scrollHeight;
				}		
				function clearLog(message) {
					var r = document.getElementById('results');
					r.innerHTML = message + '<br>';
					r.scrollTop = r.scrollHeight;
				}
		
			}

		}
	);
	

	
	jQuery('#icdash_load_incremental').on
	(
		{

			click: function(e) 
			{
				
				/*
				when the button is clicked, we start from the current month and start loading them one month at a time
				we then go back one month and load another set of data
				*/
				jQuery("#icdash-progress-parent").show("slow");
				console.log(icdash_global["install_complete"]);
				
		
		
				var v_urls=icdash_global["data_load_url"];		 //these urls are passed by the plugin main page that generates the config page  
				var v_current_url=0;		   
				icdash_clearLog("Initializing...");
				$("#result_header1").html("Initiating Data Processing...Please wait");
		
				
		
			  var newEtlDate=$("#icdash_etl_start_date").val();
			  var newEtlIncrementDays=$("#icdash_etl_increment_days").val();
			  var postUrl=v_urls[0]+"?mode=set_job_start_date";
		
			  icdash_setIncrementalJobDate(postUrl,newEtlDate,newEtlIncrementDays);

			} //END jQuery('#icdash_load_incremental')
		

		}
	);
	

		function icdash_setIncrementalJobDate(pURL,pStartDate,pIncrementDays)
		{
		  var dataStr="start_date="+pStartDate+"&increment_days="+pIncrementDays;
		  var postUrl=pURL;
	  
		  
			 $.post(postUrl
					, {start_date : pStartDate, increment_days  : pIncrementDays}
					, function(datax,  textStatus, jqXHR)
						{
							try
							{
								if (datax['status']=="SUCCESS")
								{					
									icdash_global["UI_JOB"]= datax;
									icdash_addLog("Initiating incremental load... Please wait");
									icdash_startIncrementalLoad();
									return;
								}
								else if (datax['status']=="ERROR")
								{					
									icdash_addLog(datax['error_mesg']);
									icdash_addLog(" Error during "+postUrl+" ... please check log files in the <strong>infocaptor/logs</strong> subdirectory "); 
									return;
								}
								else
								{
								icdash_global["UI_JOB"]= datax;
								//do something more..update message box etc
								return;
								}
							}	
							catch(error)
							{
								icdash_addLog(" Error during "+pURL+" ... please check logs "); 
								icdash_addLog('Possibly your PHP session has timed out. Please adjust or increase your timeout and restart the install ');								
							}								
						}
					, "json"
			);			
		}

		function icdash_getIncrementalJobDetails(pURL)
		{
		  var dataStr="x=y";
		  var postUrl=pURL;
		  console.log(postUrl);
			 $.post(postUrl
					, dataStr
					, function(datax,  textStatus, jqXHR)
						{
							try
							{
								if (datax['status']=="SUCCESS")
								{					
									icdash_global["UI_JOB"]= datax;
									return;
								}
								else if (datax['status']=="ERROR")
								{					
									icdash_addLog(datax['error_mesg']);
									icdash_addLog(" Error during "+postUrl+" ... please check log files in the <strong>infocaptor/logs</strong> subdirectory "); 
									return;
								}								
								else
								{
								icdash_global["UI_JOB"]= datax;
								//do something more..update message box etc
								return;
								}
							}	
							catch(error)
							{
								icdash_addLog(" Error during "+pURL+" ... please check logs ");
							}								
						}
					, "json"
			);			
			
		}	
		function icdash_updateOptions()
		{
			var b =  $(this).serialize();
			$.post( 'options.php', b );
		}
		
		/**kick off the load**/
		function icdash_startIncrementalLoad()
		{
		   if (icdash_global["UI_JOB"]["job_details"]["load_status"]=="complete")
		   {
				icdash_addLog("History data load complete"); 
				$( "#icdash_dialog" ).text("Initial data processing is complete. Would you like to send the processing times to InfoCaptor? This information will help us fine tune the processing logic");
				$( "#icdash_dialog" ).dialog({
					modal: true,
					buttons: {
					  Ok: function() {
						  var log_text=encodeURIComponent($("#results").text());
						window.open('mailto:contact+wooinfo@infocaptor.com?subject=WooCommerce Data Processing Times&body='+log_text, '_blank'); 
						$( this ).dialog( "close" );
					  }
					}
				  });	
				return;
		   }

			var dataStr="x=y";
			var v_urls=icdash_global["data_load_url"];		 //these urls are passed by the plugin main page that generates the config page  
			var v_current_url=0;		   
			
			
			
			icdash_addLog("Processing data from "+icdash_global["UI_JOB"]["job_details"]["load_from_date"]
			+" to "+icdash_global["UI_JOB"]["job_details"]["load_to_date"]
			); 
			var postUrl=v_urls[0]+"?mode=run_ui_incremental_load";
			 $.post(postUrl
					, dataStr
					, function(datax,  textStatus, jqXHR)
						{
							try
							{
								if (datax['status']=="SUCCESS")
								{					
									icdash_global["UI_JOB"]= datax;
									icdash_addLog(" &nbsp;&nbsp;-- "+icdash_global["UI_JOB"]["job_details"]["last_run_rows"]+" rows processed in "+(icdash_global["UI_JOB"]["execution_time"]).toFixed(2)+" seconds"); 
									
									icdash_startIncrementalLoad(); /** if not done then keep kicking off the load **/
									return;
								}
								else if (datax['status']=="ERROR")
								{					
									icdash_addLog(datax['error_mesg']);
									icdash_addLog(" Error during "+postUrl+" ... please check log files in the <strong>infocaptor/logs</strong> subdirectory ");
									return;
								}								
								else
								{
								icdash_global["UI_JOB"]= datax;
								//do something more..update message box etc
								return;
								}	
							}
							catch(error)
							{
								icdash_addLog(" Error during "+postUrl+" ... please check logs "); 															
							}
						}
					, "json"
			);			
		}
		
		function icdash_addLog(message) {
			var r = document.getElementById('results');
			r.innerHTML += message + '<br>\r\n';
			r.scrollTop = r.scrollHeight;
		}		
		function icdash_clearLog(message) {
			var r = document.getElementById('results');
			r.innerHTML = message + '<br>\r\n';
			r.scrollTop = r.scrollHeight;
		}
				
})( jQuery );