<?php

/**
 * Fired during plugin activation
 *
 * @link       https://www.infocaptor.com
 * @since      1.0.0
 *
 * @package    Dash_Reports_Woocommerce_Ic
 * @subpackage Dash_Reports_Woocommerce_Ic/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Dash_Reports_Woocommerce_Ic
 * @subpackage Dash_Reports_Woocommerce_Ic/includes
 * @author     Nilesh Jethwa <contact@infocaptor.com>
 */
class Dash_Reports_Woocommerce_Ic_Activator {

	/**
	 * Short Description. (use period)
	 *
	 * Long Description.
	 *
	 * @since    1.0.0
	 */
	public static function activate() {
	
		$key = get_option('icdash_woo_key');
		
		if($key === false)
		{
			$key = "INFOCAPTOR_WOOCOMMERCE_REPORTING";
			
			update_option('icdash_woo_key', $key);
			update_option('icdash_etl_start_date', date("Y-m-d", strtotime("-1 year", time())));
			update_option('icdash_etl_increment_days', 30);
				
		}
		//static::setup_infocaptor_rewrite();
		static::setup_infocaptor_db();

	}
	
	public static function setup_infocaptor_rewrite()
	{
		//add_action('init', 'icdash_woo_add_rewrite_rule');
    } 


	
	public static function setup_infocaptor_db()
	{

		require_once plugin_dir_path( __FILE__ ) . '/class-infocaptor-db.php';	
		include plugin_dir_path( __FILE__ )."/../infocaptor/system/property_definition.php";
		
		global $icdb;
		global $wpdb;

		//create the needed upload subdirectories
			$upload = wp_upload_dir();
			$upload_dir = $upload['basedir'];
			$upload_sub_dir = wp_normalize_path($upload_dir . '/infocaptor/data/admin');
			

			if ( ! file_exists( $upload_sub_dir ) ) 
			{
				
				if (wp_mkdir_p( $upload_sub_dir )=== TRUE)
				{
					//director exists or created
					//if not then use alternative loc
				}
				else
				{
					//alternative location
				}
			}

		$icdb = new infoDB($wpdb);
		$dbhost=$icdb->getDbHost();
		$dbport="3306";
		$dbuser=$icdb->getDbUser();
		$dbpass=$icdb->getDbPass();
		$dbname=$icdb->getDbName();
		$content_path=content_url();	
   
		$d="$";	  
		$db_file=<<<SOD
<?php
//db details set using wpdb

{$d}xdb_array['repository']['type']="mysql";
{$d}xdb_array['repository']['host']="{$dbhost}";
{$d}xdb_array['repository']['port']="{$dbport}";
{$d}xdb_array['repository']['user']="{$dbuser}";
{$d}xdb_array['repository']['pass']="{$dbpass}";
{$d}xdb_array['repository']['name']="{$dbname}";
{$d}xdb_array['repository']['table_prefix']="{$wpdb->prefix}";
{$d}xdb_array['repository']['wpcontent']="{$content_path}";


?>
SOD;

		$result=$property['tofile']("{$property['dbconfig_location']}",$db_file);		//xmember dbconfig so that when designer is used it knows where to connect
		
		$admin_api_id=sha1("NONE1"."23xcER920DJFUInfoCaptor");

		$key=$dbuser.substr($admin_api_id, 1, 11);
		$encrypted = static::xa_encrypt_decrypt("encrypt",$key, $dbpass); 	
		$db_file=<<<SOD
<?php
//db details set using wpdb

{$d}db_param["type"]='mysql';
{$d}db_param["driver_str"]='';
{$d}db_param["dbname"]='{$dbname}';
{$d}db_param["host"]='{$dbhost}';
{$d}db_param["port"]='{$dbport}';
{$d}db_param["dbuser"]='{$dbuser}';
{$d}db_param["dbpass"]='{$encrypted}';
{$d}db_param["connection_description"]='woocommerce database';
{$d}db_param['table_prefix']="{$wpdb->prefix}";
{$d}db_param['wpcontent']="{$content_path}";


?>
SOD;
		$result=$property['tofile']("{$property['data_dir']}/admin/woocommerce_db.php",$db_file);		//woo handle so the dashboards can connect
	}

	public static function  xa_encrypt_decrypt($action,$key, $string) 
	{
		$output = false;
		$encrypt_method = "AES-256-CBC";
		$secret_key = $key." key";
		$secret_iv = $key." iv";
		// hash
		$key = hash('sha256', $secret_key);
		
		// iv - encrypt method AES-256-CBC expects 16 bytes - else you will get a warning
		$iv = substr(hash('sha256', $secret_iv), 0, 16);
		if ( $action == 'encrypt' ) {
			$output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
			$output = base64_encode($output);
		} else if( $action == 'decrypt' ) {
			$output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
		}
		return $output;
	}
	
	
}



/*
function icdash_woo_add_rewrite_rule()
{
	add_rewrite_rule('^infocaptor_view/([A-Za-z0-9_]+)/([A-Za-z0-9_]+)/?$', 'wp-content/plugins/dash-reports-woocommerce-ic/infocaptor/mt.php?pr=$matches[1]&pa=$matches[2]&v=__', 'top');
	flush_rewrite_rules();
} 
*/